import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 11.10.16
 */
public class TestGen {

    private final boolean FOR_POLYGON = true;

    public static final int MAX_N = 20_000;
    public static final int MAX_BITS = 12;
    public static final int MAX_VAL = 1<<MAX_BITS;

    int testCount;
    static Random random = new Random(0x101);

    private String folder;
    boolean noModifyTest = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void println(PrintWriter writer, int... arr) {
        for (int i = 0; i < arr.length; ++i)
            if (i + 1 == arr.length) writer.println(arr[i]);
            else writer.print(arr[i] + " ");
    }

    private void print(Object... objects) throws IOException {
        print((int[])objects[0], (int)objects[1], (String)objects[2]);
    }

    public void print(int[] a, int k, String descr) throws IOException {
        String path = (FOR_POLYGON ?
                String.format("%d",++testCount) :
                String.format("../%s/%02d", folder, ++testCount));

        PrintWriter out = new PrintWriter(path);
        System.out.println("Generating test #" + testCount + ": " + descr);
        int n = a.length;
        out.println(n + " " + k);
        println(out, a);
        out.close();
    }

    public void genSamples() throws IOException {
        setFolder("tests");
        noModifyTest = true;
        System.out.println("==== Generating samples ===");
        print(new int[]{5, 4, 3}, 1, "sample1");
        print(new int[]{5, 4, 3}, 2, "sample2");
        print(new int[]{6, 12, 7, 8, 5, 13}, 3, "sample3");
        noModifyTest = false;
    }

    void genHandleTests() throws IOException {
        noModifyTest = true;
        System.out.println("==== Generating handle tests ====");
        print(new int[]{1}, 1, "n=1");
        print(new int[]{0}, 1, "n=1");
        print(new int[]{5, 3, 2, 8}, 2, "n=4");
        noModifyTest = false;
    }

    void genTests() throws IOException {
        System.out.println("==== Generating tests ====");
        print(genNo(MAX_N, 11));
        print(genNo(MAX_N, 11));
        print(genNo(MAX_N, 10));
        print(genNo(MAX_N, 5));
        print(genNo(MAX_N, MAX_N / 2));
        print(new Object[]{oneZero(MAX_N), 1, "one zero"});
        print(goldNumber(MAX_N));
        print(goldNumber(MAX_N));
        print(goldNumber(MAX_N));
        print(genRandom(MAX_N, 11));
        print(genRandom(MAX_N, 10));
        print(genRandom(MAX_N, 9));
        print(genRandom(MAX_N, 8));
        print(genRandom(MAX_N, 7));
        print(genRandom(MAX_N, 5));
        print(genRandom(MAX_N, 1));

        print(genRandomNo(MAX_N, 11));
        print(genRandomNo(MAX_N, 10));
        print(genRandomNo(MAX_N, 9));
        print(genRandomNo(MAX_N, 8));
        print(genRandomNo(MAX_N, 7));
        print(genRandomNo(MAX_N, 5));
        print(genRandomNo(MAX_N, 1));
    }

    private int numWithBits(int k) {
        int[] bits = genCombine(12, k);
        int num = 0;
        for (int bit : bits) {
            num |= 1 << (bit - 1);
        }
        return num;
    }

    private int[] randArrBits(int n, int k) {
        int[] a = new int[n];
        for (int i = 0; i < n; ++i) {
            a[i] = numWithBits(k);
        }
        return a;
    }

    private Object[] genRandom(int n, int bits) {
        int[] a = randArrBits(n, bits);
        int k = getMinK(a);
        return new Object[]{a, k, "random yes test, answer = " + k};
    }

    private Object[] genRandomNo(int n, int bits) {
        int[] a = randArrBits(n, bits);
        int k = getMinK(a) - 1;
        return new Object[]{a, k, "random no test, answer = " + (k + 1)};
    }

    private int randBit() {
        return randInt(0, MAX_BITS - 1);
    }

    private Object[] goldNumber(int n) {
        int part = (n - 1) / 3;
        int bit1 = randBit();
        int bit2 = randBit();
        while (bit2 == bit1) {
            bit2 = randBit();
        }
        int[] a = noBit(part, bit1);
        int[] b = noBit(part, bit2);
        int[] c = noBit(part, bit1, bit2);
        int x = ((1<<MAX_BITS)-1)&(~noBitX(bit1, bit2));
        int[] ret = new int[a.length + b.length + c.length + 1];
        System.arraycopy(a, 0, ret, 0, a.length);
        System.arraycopy(b, 0, ret, a.length, b.length);
        System.arraycopy(c, 0, ret, a.length + b.length, c.length);
        ret[a.length + b.length + c.length] = x;
        shuffle(ret, 0, ret.length);
        int k = getMinK(a);
        if (k > n) {
            throw new IllegalStateException("Lol, k > n");
        }
        return new Object[]{ret, k, "gold number test"};
    }

    private int[] oneZero(int n) {
        int[] a = new int[n];
        for (int i = 0; i < n; ++i) {
            a[i] = randBit();
        }
        int i = randInt(0, n - 1);
        a[i] = 0;
        return a;
    }

    private int getMinK(int[] a) {
        int n = a.length;
        int[] dp = new int[MAX_VAL];
        Arrays.fill(dp, n + 1);
        dp[MAX_VAL - 1] = 0;
        for (int x : a)
            for (int ms = 0; ms < MAX_VAL; ++ms) {
                dp[ms & x] = Math.min(dp[ms & x], dp[ms] + 1);
            }
        return dp[0];
    }

    private Object[] genNo(int n, int k) {
        int bit = randBit();
        int[] a = noBit(n, bit);
        return new Object[]{a, k, "gen no, no bit"};
    }

    private int[] noBit(int n, int... bits) {
        int[] a = new int[n];
        for (int i = 0; i < n; ++i) {
            a[i] = noBitX(bits);
        }
        return a;
    }

    private int noBitX(int... bits) {
        while (true) {
            boolean fail = false;
            int x = randInt(0, MAX_VAL - 1);
            for (int bit : bits)
                if (((x >> bit) & 1) == 0) {
                    fail = true;
                    break;
                }
            if (!fail) {
                return x;
            }
        }
    }



    private void swap(int[] a, int i, int j) {
        int t = a[i];
        a[i] = a[j];
        a[j] = t;
    }

    public void run() throws IOException {
        if (!FOR_POLYGON) {
            new File("../tests").mkdir();
        }

        genSamples();
        genHandleTests();
        genTests();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }

    private int[] genCombine(int n, int k) {
        if (k > n)
            throw new IllegalArgumentException("k > n");

        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            p.add(i + 1);
        shuffle(p);
        int[] ret = new int[k];
        List<Integer> sublist = p.subList(0, k);
        for (int i = 0; i < k; ++i)
            ret[i] = sublist.get(i);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genPartitionViaCombine(int n, int[] comb) {
        int k = comb.length + 1;
        int[] ret = new int[k];
        if (k >= 2)
            ret[0] = comb[0];

        for (int i = 1; i + 1 < k; ++i)
            ret[i] = comb[i] - comb[i - 1];
        ret[k - 1] = k >= 2 ?  n - comb[k - 2] : n;
        return ret;
    }

    public int[] genPartition(int sum, int k) {
        int[] comb = genCombine(sum - 1, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genPartitionWithZeroes(int sum, int k) {
        int[] comb = genCombineWithEq(sum, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genCombineWithEq(int n, int k) {
        int[] ret = new int[k];
        for (int i = 0; i < k; ++i)
            ret[i] = randInt(0, n);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genRandPerm(int n) {
        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i) {
            p.add(i);
        }
        Collections.shuffle(p, random);
        int[] ret = new int[n];
        for (int i = 0; i < n; ++i) {
            ret[i] = p.get(i);
        }
        return ret;
    }

    private void shuffle(List<?> a) {
        Collections.shuffle(a, random);
    }

    private void shuffle(int[] a, int from, int to) {
        int[] b = Arrays.copyOfRange(a, from, to);
        Integer[] bInteger = new Integer[b.length];
        for (int i = 0; i < b.length; ++i)
            bInteger[i] = b[i];

        List<Integer> arr = Arrays.asList(bInteger);
        shuffle(arr);
        for (int i = from; i < to; ++i)
            a[i] = arr.get(i - from);
    }

    private int[] toIntArray(Integer[] a) {
        int[] ret = new int[a.length];
        for (int i = 0; i < ret.length; ++i)
            ret[i] = a[i];
        return ret;
    }


    private static int randInt(int l, int r) {
        return random.nextInt(r - l + 1) + l;
    }

    private static int randInt(int n) {
        return random.nextInt(n) + 1;
    }
}
