import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 11.10.16
 */
public class TestGen {

    private final boolean FOR_POLYGON = true;

    public static final int MAX_N = 1000;
    public static final int MAX_VAL = 1_000_000_000;

    int testCount;
    static Random random = new Random(0x101);

    private String folder;
    boolean noModifyTest = false;

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void println(PrintWriter writer, int... arr) {
        for (int i = 0; i < arr.length; ++i)
            if (i + 1 == arr.length) writer.println(arr[i]);
            else writer.print(arr[i] + " ");
    }

    public void print(int[] a, String descr) throws IOException {
        String path = (FOR_POLYGON ?
                String.format("%d",++testCount) :
                String.format("../%s/%02d", folder, ++testCount));

        PrintWriter out = new PrintWriter(path);
        System.out.println("Generating test #" + testCount + ": " + descr);
        int n = a.length;
        out.println(n);
        println(out, a);
        out.close();
    }

    public void genSamples() throws IOException {
        setFolder("tests");
        noModifyTest = true;
        System.out.println("==== Generating samples ===");
        print(new int[]{2, 1}, "sample1");
        print(new int[]{1, 2, 3}, "sample2");
        print(new int[]{2, 1, 4, 6}, "sample3");
        noModifyTest = false;
    }

    void genHandleTests() throws IOException {
        noModifyTest = true;
        System.out.println("==== Generating handle tests ====");
        print(new int[]{1, 2}, "n=2");
        print(new int[]{2, 1}, "n=2");
        print(new int[]{1, 2, 3}, "n=3");
        print(new int[]{2, 1, 3}, "n=3");
        print(new int[]{2, 1, 3}, "n=3");
        print(new int[]{2, 2, 4, 6}, "n=4");
        print(new int[]{10, 2, 12, 4, 5, 6}, "n=6");
        print(new int[]{1, 2, 3, 5, 5, 9, 8, 8}, "n=8");
        noModifyTest = false;
    }

    void genTests() throws IOException {
        System.out.println("==== Generating tests ====");
        print(yesRandSwap(randInt(MAX_N), 0), "good array");
        print(yesRandSwap(randInt(MAX_N), 0), "good array");
        print(yesRandSwap(randInt(MAX_N), 1), "random swap");
        print(yesRandSwap(randInt(MAX_N), 1), "random swap");
        print(yesRandSwap(randInt(MAX_N), 1), "random swap");
        print(yesRandSwap(randInt(MAX_N), 1), "random swap");
        print(yesRandSwap(randInt(MAX_N), 2), "two random swaps");
        print(yesRandSwap(MAX_N, 0), "0 random swaps, n = MAX_N");
        print(yesRandSwap(MAX_N, 1), "1 random swaps, n = MAX_N");
        print(yesRandSwap(MAX_N, 2), "2 random swaps, n = MAX_N");
        print(yesRandSwap(MAX_N, 100), "100 random swaps");
        print(fullRandSwap(randInt(MAX_N)), "");
    }

    private int[] yesRandSwap(int n, int swaps) {
        int[] a = new int[n];
        for (int i = 0; i < n; ++i) {
            int x = randInt(MAX_VAL / 2 - 1);
            a[i] = 2 * x + (i % 2 == 0 ? 1 : 0);
        }

        for (int it = 0; it < swaps; ++it) {
            int i = randInt(0, n - 1);
            int j = randInt(0, n - 1);
            while (i == j) {
                j = randInt(0, n - 1);
            }
            swap(a, i, j);
        }
        return a;
    }

    private int[] fullRandSwap(int n) {
        int[] a = yesRandSwap(n, 0);
        for (int i = 0; i < n; ++i) {
            a[i]++;
        }
        return a;
    }

    private void swap(int[] a, int i, int j) {
        int t = a[i];
        a[i] = a[j];
        a[j] = t;
    }

    public void run() throws IOException {
        if (!FOR_POLYGON) {
            new File("../tests").mkdir();
        }

        genSamples();
        genHandleTests();
        genTests();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }

    private int[] genCombine(int n, int k) {
        if (k > n)
            throw new IllegalArgumentException("k > n");

        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            p.add(i + 1);
        shuffle(p);
        int[] ret = new int[k];
        List<Integer> sublist = p.subList(0, k);
        for (int i = 0; i < k; ++i)
            ret[i] = sublist.get(i);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genPartitionViaCombine(int n, int[] comb) {
        int k = comb.length + 1;
        int[] ret = new int[k];
        if (k >= 2)
            ret[0] = comb[0];

        for (int i = 1; i + 1 < k; ++i)
            ret[i] = comb[i] - comb[i - 1];
        ret[k - 1] = k >= 2 ?  n - comb[k - 2] : n;
        return ret;
    }

    public int[] genPartition(int sum, int k) {
        int[] comb = genCombine(sum - 1, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genPartitionWithZeroes(int sum, int k) {
        int[] comb = genCombineWithEq(sum, k - 1);
        return genPartitionViaCombine(sum, comb);
    }

    private int[] genCombineWithEq(int n, int k) {
        int[] ret = new int[k];
        for (int i = 0; i < k; ++i)
            ret[i] = randInt(0, n);
        Arrays.sort(ret);
        return ret;
    }

    private int[] genRandPerm(int n) {
        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i) {
            p.add(i);
        }
        Collections.shuffle(p, random);
        int[] ret = new int[n];
        for (int i = 0; i < n; ++i) {
            ret[i] = p.get(i);
        }
        return ret;
    }

    private void shuffle(List<?> a) {
        Collections.shuffle(a, random);
    }

    private void shuffle(int[] a, int from, int to) {
        int[] b = Arrays.copyOfRange(a, from, to);
        Integer[] bInteger = new Integer[b.length];
        for (int i = 0; i < b.length; ++i)
            bInteger[i] = b[i];

        List<Integer> arr = Arrays.asList(bInteger);
        shuffle(arr);
        for (int i = from; i < to; ++i)
            a[i] = arr.get(i - from);
    }

    private int[] toIntArray(Integer[] a) {
        int[] ret = new int[a.length];
        for (int i = 0; i < ret.length; ++i)
            ret[i] = a[i];
        return ret;
    }


    private static int randInt(int l, int r) {
        return random.nextInt(r - l + 1) + l;
    }

    private static int randInt(int n) {
        return random.nextInt(n) + 1;
    }
}
